"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.Zoom = void 0;
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _raCore = require("@ramap/ra-core");
var _raUtils = require("@ramap/ra-utils");
var _icon = require("../utils/icon");
var _baseControl = require("./baseControl");
class Zoom extends _baseControl.Control {
  constructor(...args) {
    super(...args);
    (0, _defineProperty2.default)(this, "disabled", void 0);
    (0, _defineProperty2.default)(this, "zoomInButton", void 0);
    (0, _defineProperty2.default)(this, "zoomOutButton", void 0);
    (0, _defineProperty2.default)(this, "zoomNumDiv", void 0);
    (0, _defineProperty2.default)(this, "zoomIn", () => {
      if (!this.disabled && this.mapsService.getZoom() < this.mapsService.getMaxZoom()) {
        this.mapsService.zoomIn();
      }
    });
    (0, _defineProperty2.default)(this, "zoomOut", () => {
      if (!this.disabled && this.mapsService.getZoom() > this.mapsService.getMinZoom()) {
        this.mapsService.zoomOut();
      }
    });
    (0, _defineProperty2.default)(this, "updateDisabled", () => {
      const mapsService = this.mapsService;
      this.zoomInButton.removeAttribute('disabled');
      this.zoomOutButton.removeAttribute('disabled');
      if (this.disabled || mapsService.getZoom() <= mapsService.getMinZoom()) {
        this.zoomOutButton.setAttribute('disabled', 'true');
      }
      if (this.controlOption.showZoom && this.zoomNumDiv) {
        this.zoomNumDiv.innerText = String(Math.floor(mapsService.getZoom()));
      }
      if (this.disabled || mapsService.getZoom() >= mapsService.getMaxZoom()) {
        this.zoomInButton.setAttribute('disabled', 'true');
      }
    });
  }
  getDefault(option) {
    return (0, _objectSpread2.default)((0, _objectSpread2.default)({}, super.getDefault(option)), {}, {
      position: _raCore.PositionType.BOTTOMRIGHT,
      name: 'zoom',
      zoomInText: (0, _icon.createRaIcon)('ra-icon-enlarge'),
      zoomInTitle: 'Zoom in',
      zoomOutText: (0, _icon.createRaIcon)('ra-icon-narrow'),
      zoomOutTitle: 'Zoom out',
      showZoom: false
    });
  }
  setOptions(newOptions) {
    super.setOptions(newOptions);
    if (this.checkUpdateOption(newOptions, ['zoomInText', 'zoomInTitle', 'zoomOutText', 'zoomOutTitle', 'showZoom'])) {
      this.resetButtonGroup(this.container);
    }
  }
  onAdd() {
    const container = _raUtils.DOM.create('div', 'ra-control-zoom');
    this.resetButtonGroup(container);
    this.mapsService.on('zoomend', this.updateDisabled);
    this.mapsService.on('zoomchange', this.updateDisabled);
    return container;
  }
  onRemove() {
    this.mapsService.off('zoomend', this.updateDisabled);
    this.mapsService.off('zoomchange', this.updateDisabled);
  }
  disable() {
    this.disabled = true;
    this.updateDisabled();
    return this;
  }
  enable() {
    this.disabled = false;
    this.updateDisabled();
    return this;
  }
  resetButtonGroup(container) {
    _raUtils.DOM.clearChildren(container);
    this.zoomInButton = this.createButton(this.controlOption.zoomInText, this.controlOption.zoomInTitle, 'ra-button-control', container, this.zoomIn);
    if (this.controlOption.showZoom) {
      this.zoomNumDiv = this.createButton('0', '', 'ra-button-control ra-control-zoom__number', container);
    }
    this.zoomOutButton = this.createButton(this.controlOption.zoomOutText, this.controlOption.zoomOutTitle, 'ra-button-control', container, this.zoomOut);
    this.updateDisabled();
  }
  createButton(html, tile, className, container, fn) {
    const link = _raUtils.DOM.create('button', className, container);
    if (typeof html === 'string') {
      link.innerHTML = html;
    } else {
      link.append(html);
    }
    link.title = tile;
    if (fn) {
      link.addEventListener('click', fn);
    }
    return link;
  }
}
exports.default = exports.Zoom = Zoom;